env: allow ENV_IS_NOWHERE with other storage target
[oweals/u-boot.git] / env / env.c
index e033b461244808b81df25aa0e02785e5f3c30463..4b417b90a2912c201f201514e840d644f03e45e7 100644 (file)
--- a/env/env.c
+++ b/env/env.c
@@ -71,6 +71,9 @@ static enum env_location env_locations[] = {
 #ifdef CONFIG_ENV_IS_IN_REMOTE
        ENVL_REMOTE,
 #endif
+#ifdef CONFIG_ENV_IS_IN_SATA
+       ENVL_ESATA,
+#endif
 #ifdef CONFIG_ENV_IS_IN_SPI_FLASH
        ENVL_SPI_FLASH,
 #endif
@@ -174,6 +177,7 @@ int env_get_char(int index)
 int env_load(void)
 {
        struct env_driver *drv;
+       int best_prio = -1;
        int prio;
 
        for (prio = 0; (drv = env_driver_lookup(ENVOP_LOAD, prio)); prio++) {
@@ -186,22 +190,38 @@ int env_load(void)
                        continue;
 
                printf("Loading Environment from %s... ", drv->name);
+               /*
+                * In error case, the error message must be printed during
+                * drv->load() in some underlying API, and it must be exactly
+                * one message.
+                */
                ret = drv->load();
-               if (ret)
-                       printf("Failed (%d)\n", ret);
-               else
+               if (!ret) {
                        printf("OK\n");
-
-               if (!ret)
                        return 0;
+               } else if (ret == -ENOMSG) {
+                       /* Handle "bad CRC" case */
+                       if (best_prio == -1)
+                               best_prio = prio;
+               } else {
+                       debug("Failed (%d)\n", ret);
+               }
        }
 
        /*
         * In case of invalid environment, we set the 'default' env location
* to the highest priority. In this way, next calls to env_save()
* will restore the environment at the right place.
         */
-       env_get_location(ENVOP_LOAD, 0);
+       if (best_prio >= 0)
+               debug("Selecting environment with bad CRC\n");
+       else
+               best_prio = 0;
+       env_get_location(ENVOP_LOAD, best_prio);
 
        return -ENODEV;
 }
@@ -186,22 +190,38 @@ int env_load(void)
        /*
         * In case of invalid environment, we set the 'default' env location
-        * to the highest priority. In this way, next calls to env_save()
-        * will restore the environment at the right place.
+        * to the best choice, i.e.:
+        *   1. Environment location with bad CRC, if such location was found
+        *   2. Otherwise use the location with highest priority
+        *
+        * This way, next calls to env_save() will restore the environment
+        * at the right place.
         */